import sys

if 'runtestcase' in sys.argv[1:]: # pragma: no cover
    import gevent
    import gevent.subprocess
    gevent.spawn(sys.exit, 'bye')
    # Look closely, this doesn't actually do anything, that's a string
    # not a division
    gevent.subprocess.Popen([sys.executable, '-c', '"1/0"'])
    gevent.sleep(1)
else:
    # XXX: Handle this more automatically. See comments in the testrunner.
    from gevent.testing.resources import exit_without_resource
    exit_without_resource('subprocess')

    import subprocess
    for _ in range(5):
        out, err = subprocess.Popen([sys.executable, '-W', 'ignore',
                                     __file__, 'runtestcase'],
                                    stderr=subprocess.PIPE).communicate()
        # We've seen a few unexpected forms of output.
        #
        # The first involves 'refs'; I don't remember what that was
        # about, but I think it had to do with debug builds of Python.
        #
        # The second is the classic "Unhandled exception in thread
        # started by \nsys.excepthook is missing\nlost sys.stderr".
        # This is a race condition between closing sys.stderr and
        # writing buffered data to a pipe that hasn't been read. We
        # only see this using GEVENT_FILE=thread (which makes sense);
        # likewise, on Python 2 with thread, we can sometimes get
        # `super() argument 1 must be type, not None`; this happens on module
        # cleanup.
        #
        # The third is similar to the second: "AssertionError:
        # ...\nIOError: close() called during concurrent operation on
        # the same file object.\n"
        if b'refs' in err or b'sys.excepthook' in err or b'concurrent' in err:
            assert err.startswith(b'bye'), repr(err) # pragma: no cover
        else:
            assert err.strip() == b'bye', repr(err)



# Copyright (c) 2008 AG Projects
# Author: Denis Bilenko
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.

import gevent.testing as greentest
import gevent
from gevent import util, socket

DELAY = 0.1


class Test(greentest.TestCase):

    @greentest.skipOnAppVeyor("Timing causes the state to often be [start,finished]")
    def test_killing_dormant(self):
        state = []

        def test():
            try:
                state.append('start')
                gevent.sleep(DELAY * 3.0)
            except: # pylint:disable=bare-except
                state.append('except')
                # catching GreenletExit

            state.append('finished')

        g = gevent.spawn(test)
        gevent.sleep(DELAY / 2)
        assert state == ['start'], state
        g.kill()
        # will not get there, unless switching is explicitly scheduled by kill
        self.assertEqual(state, ['start', 'except', 'finished'])

    def test_nested_with_timeout(self):
        def func():
            return gevent.with_timeout(0.2, gevent.sleep, 2, timeout_value=1)
        self.assertRaises(gevent.Timeout, gevent.with_timeout, 0.1, func)

    def test_sleep_invalid_switch(self):
        p = gevent.spawn(util.wrap_errors(AssertionError, gevent.sleep), 2)
        gevent.sleep(0)  # wait for p to start, because actual order of switching is reversed
        switcher = gevent.spawn(p.switch, None)
        result = p.get()
        assert isinstance(result, AssertionError), result
        assert 'Invalid switch' in str(result), repr(str(result))
        switcher.kill()

    if hasattr(socket, 'socketpair'):

        def _test_wait_read_invalid_switch(self, sleep):
            sock1, sock2 = socket.socketpair()
            try:
                p = gevent.spawn(util.wrap_errors(AssertionError,
                                                  socket.wait_read), # pylint:disable=no-member
                                 sock1.fileno())
                gevent.get_hub().loop.run_callback(switch_None, p)
                if sleep is not None:
                    gevent.sleep(sleep)
                result = p.get()
                assert isinstance(result, AssertionError), result
                assert 'Invalid switch' in str(result), repr(str(result))
            finally:
                sock1.close()
                sock2.close()

        def test_invalid_switch_None(self):
            self._test_wait_read_invalid_switch(None)

        def test_invalid_switch_0(self):
            self._test_wait_read_invalid_switch(0)

        def test_invalid_switch_1(self):
            self._test_wait_read_invalid_switch(0.001)

        # we don't test wait_write the same way, because socket is always ready to write


def switch_None(g):
    g.switch(None)


class TestTimers(greentest.TestCase):

    def test_timer_fired(self):
        lst = [1]

        def func():
            gevent.spawn_later(0.01, lst.pop)
            gevent.sleep(0.02)

        gevent.spawn(func)
        # Func has not run yet
        self.assertEqual(lst, [1])
        # Run callbacks but don't yield.
        gevent.sleep()

        # Let timers fire. Func should be done.
        gevent.sleep(0.1)
        self.assertEqual(lst, [])


    def test_spawn_is_not_cancelled(self):
        lst = [1]

        def func():
            gevent.spawn(lst.pop)
            # exiting immediately, but self.lst.pop must be called
        gevent.spawn(func)
        gevent.sleep(0.1)
        self.assertEqual(lst, [])


if __name__ == '__main__':
    greentest.main()



import unittest
import warnings
import sys

# All supported python versions now provide SSLContext.
# We import it by name and subclass it here by name.
# compare with warning3.py
from ssl import SSLContext

class MySubclass(SSLContext):
    pass

# This file should only have this one test in it
# because we have to be careful about our imports
# and because we need to be careful about our patching.

class Test(unittest.TestCase):

    @unittest.skipIf(sys.version_info[:2] < (3, 6),
                     "Only on Python 3.6+")
    def test_ssl_subclass_and_module_reference(self):

        from gevent import monkey

        self.assertFalse(monkey.saved)

        with warnings.catch_warnings(record=True) as issued_warnings:
            warnings.simplefilter('always')

            monkey.patch_all()
            monkey.patch_all()

        issued_warnings = [x for x in issued_warnings
                           if isinstance(x.message, monkey.MonkeyPatchWarning)]

        self.assertEqual(1, len(issued_warnings))
        message = issued_warnings[0].message
        self.assertIn("Modules that had direct imports", str(message))
        self.assertIn("Subclasses (NOT patched)", str(message))



if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-
# copyright 2018 gevent
"""
Exceptions.

.. versionadded:: 1.3b1

"""
from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

from greenlet import GreenletExit

__all__ = [
    'LoopExit',
]


class LoopExit(Exception):
    """
    Exception thrown when the hub finishes running (`gevent.hub.Hub.run`
    would return).

    In a normal application, this is never thrown or caught
    explicitly. The internal implementation of functions like
    :meth:`gevent.hub.Hub.join` and :func:`gevent.joinall` may catch it, but user code
    generally should not.

    .. caution::
       Errors in application programming can also lead to this exception being
       raised. Some examples include (but are not limited too):

       - greenlets deadlocking on a lock;
       - using a socket or other gevent object with native thread
         affinity from a different thread

    """

    @property
    def hub(self):
        """
        The (optional) hub that raised the error.

        .. versionadded:: 20.12.0
        """
        # XXX: Note that semaphore.py does this manually.
        if len(self.args) == 3: # From the hub
            return self.args[1]

    def __repr__(self):
        # pylint:disable=unsubscriptable-object
        if len(self.args) == 3: # From the hub
            import pprint
            return (
                "%s\n"
                "\tHub: %s\n"
                "\tHandles:\n%s"
            ) % (
                self.args[0],
                self.args[1],
                pprint.pformat(self.args[2])
            )
        return Exception.__repr__(self)

    def __str__(self):
        return repr(self)

class BlockingSwitchOutError(AssertionError):
    """
    Raised when a gevent synchronous function is called from a
    low-level event loop callback.

    This is usually a programming error.
    """


class InvalidSwitchError(AssertionError):
    """
    Raised when the event loop returns control to a greenlet in an
    unexpected way.

    This is usually a bug in gevent, greenlet, or the event loop.
    """

class ConcurrentObjectUseError(AssertionError):
    """
    Raised when an object is used (waited on) by two greenlets
    independently, meaning the object was entered into a blocking
    state by one greenlet and then another while still blocking in the
    first one.

    This is usually a programming error.

    .. seealso:: `gevent.socket.wait`
    """

class InvalidThreadUseError(RuntimeError):
    """
    Raised when an object is used from a different thread than
    the one it is bound to.

    Some objects, such as gevent sockets, semaphores, and threadpools,
    are tightly bound to their hub and its loop. The hub and loop
    are not thread safe, with a few exceptions. Attempting to use
    such objects from a different thread is an error, and may cause
    problems ranging from incorrect results to memory corruption
    and a crashed process.

    In some cases, gevent catches this "accidentally", and the result is
    a `LoopExit`. In some cases, gevent doesn't catch this at all.

    In other cases (typically when the consequences are suspected to
    be more on the more severe end of the scale, and when the operation in
    question is already relatively heavyweight), gevent explicitly checks
    for this usage and will raise this exception when it is detected.

    .. versionadded:: 1.5a3
    """


class HubDestroyed(GreenletExit):
    """
    Internal exception, raised when we're trying to destroy the
    hub and we want the loop to stop running callbacks now.

    This must not be subclassed; the type is tested by identity.

    Clients outside of gevent must not raise this exception.

    .. versionadded:: 20.12.0
    """

    def __init__(self, destroy_loop):
        GreenletExit.__init__(self, destroy_loop)
        self.destroy_loop = destroy_loop



# Copyright (c) 2018 gevent community
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
from __future__ import absolute_import, print_function, division

import sys
import functools
import unittest

from . import sysinfo
from . import six

class FlakyAssertionError(AssertionError):
    "Re-raised so that we know it's a known-flaky test."

# The next exceptions allow us to raise them in a highly
# greppable way so that we can debug them later.

class FlakyTest(unittest.SkipTest):
    """
    A unittest exception that causes the test to be skipped when raised.

    Use this carefully, it is a code smell and indicates an undebugged problem.
    """

class FlakyTestRaceCondition(FlakyTest):
    """
    Use this when the flaky test is definitely caused by a race condition.
    """

class FlakyTestTimeout(FlakyTest):
    """
    Use this when the flaky test is definitely caused by an
    unexpected timeout.
    """

class FlakyTestCrashes(FlakyTest):
    """
    Use this when the test sometimes crashes.
    """

def reraiseFlakyTestRaceCondition():
    six.reraise(FlakyAssertionError,
                FlakyAssertionError(sys.exc_info()[1]),
                sys.exc_info()[2])

reraiseFlakyTestTimeout = reraiseFlakyTestRaceCondition
reraiseFlakyTestRaceConditionLibuv = reraiseFlakyTestRaceCondition
reraiseFlakyTestTimeoutLibuv = reraiseFlakyTestRaceCondition

if sysinfo.RUNNING_ON_CI or (sysinfo.PYPY and sysinfo.WIN):
    # pylint: disable=function-redefined
    def reraiseFlakyTestRaceCondition():
        # Getting stack traces is incredibly expensive
        # in pypy on win, at least in test virtual machines.
        # It can take minutes. The traceback consistently looks like
        # the following when interrupted:

        # dump_stacks -> traceback.format_stack
        #    -> traceback.extract_stack -> linecache.checkcache
        #    -> os.stat -> _structseq.structseq_new

        # Moreover, without overriding __repr__ or __str__,
        # the msg doesn't get printed like we would want (its basically
        # unreadable, all printed on one line). So skip that.

        #msg = '\n'.join(dump_stacks())
        msg = str(sys.exc_info()[1])
        six.reraise(FlakyTestRaceCondition,
                    FlakyTestRaceCondition(msg),
                    sys.exc_info()[2])

    def reraiseFlakyTestTimeout():
        msg = str(sys.exc_info()[1])
        six.reraise(FlakyTestTimeout,
                    FlakyTestTimeout(msg),
                    sys.exc_info()[2])

    if sysinfo.LIBUV:
        reraiseFlakyTestRaceConditionLibuv = reraiseFlakyTestRaceCondition
        reraiseFlakyTestTimeoutLibuv = reraiseFlakyTestTimeout


def reraises_flaky_timeout(exc_kind=AssertionError, _func=reraiseFlakyTestTimeout):

    def wrapper(f):
        @functools.wraps(f)
        def m(*args):
            try:
                f(*args)
            except exc_kind:
                _func()
        return m

    return wrapper

def reraises_flaky_race_condition(exc_kind=AssertionError):
    return reraises_flaky_timeout(exc_kind, _func=reraiseFlakyTestRaceCondition)



"""
Test the contents of the ``examples/`` directory.

If an existing test in *this* directory named ``test__example_<fn>.py`` exists,
where ``<fn>`` is the base filename of an example file, it will not be tested
here.

Examples can specify that they need particular test resources to be enabled
by commenting (one per line) ``# gevent-test-requires-resource: <resource>``;
most commonly the resource will be ``network``. You can use this technique to specify
non-existant resources for things that should never be tested.
"""
import re
import os
import glob
import time
import unittest

import gevent.testing as greentest
from gevent.testing import util

this_dir = os.path.dirname(__file__)

def _find_files_to_ignore():
    old_dir = os.getcwd()
    try:
        os.chdir(this_dir)

        result = [x[14:] for x in glob.glob('test__example_*.py')]
        if greentest.PYPY and greentest.RUNNING_ON_APPVEYOR:
            # For some reason on Windows with PyPy, this times out,
            # when it should be very fast.
            result.append("processes.py")
    finally:
        os.chdir(old_dir)

    return result

default_time_range = (2, 10)
time_ranges = {
    'concurrent_download.py': (0, 30),
    'processes.py': (0, default_time_range[-1])
}

class _AbstractTestMixin(util.ExampleMixin):
    time_range = default_time_range
    example = None

    def _check_resources(self):
        from gevent.testing import resources

        with open(os.path.join(self.cwd, self.example), 'r') as f:
            contents = f.read()

        pattern = re.compile('^# gevent-test-requires-resource: (.*)$', re.MULTILINE)
        resources_needed = re.finditer(pattern, contents)
        for match in resources_needed:
            needed = contents[match.start(1):match.end(1)]
            resources.skip_without_resource(needed)

    def test_runs(self):
        self._check_resources()

        start = time.time()
        min_time, max_time = self.time_range
        self.start_kwargs = {
            'timeout': max_time,
            'quiet': True,
            'buffer_output': True,
            'nested': True,
            'setenv': {'GEVENT_DEBUG': 'error'}
        }
        if not self.run_example():
            self.fail("Failed example: " + self.example)
        else:
            took = time.time() - start
            self.assertGreaterEqual(took, min_time)

def _build_test_classes():
    result = {}
    try:
        example_dir = util.ExampleMixin().cwd
    except unittest.SkipTest:
        util.log("WARNING: No examples dir found", color='suboptimal-behaviour')
        return result

    ignore = _find_files_to_ignore()
    for filename in glob.glob(example_dir + '/*.py'):
        bn = os.path.basename(filename)
        if bn in ignore:
            continue

        tc = type(
            'Test_' + bn,
            (_AbstractTestMixin, greentest.TestCase),
            {
                'example': bn,
                'time_range': time_ranges.get(bn, _AbstractTestMixin.time_range)
            }
        )
        result[tc.__name__] = tc
    return result

for k, v in _build_test_classes().items():
    locals()[k] = v

if __name__ == '__main__':
    greentest.main()
